import vtk

# create cuboid source
my_cuboidSource = vtk.vtkCubeSource()
my_cuboidSource.SetXLength(0.3)
my_cuboidSource.SetYLength(0.1)
my_cuboidSource.SetZLength(0.1)

# create and connect cuboid mapper
my_cuboidMapper = vtk.vtkPolyDataMapper()
my_cuboidMapper.SetInputConnection(my_cuboidSource.GetOutputPort())

# create and connect cuboid actor
my_cuboid = vtk.vtkLODActor()
my_cuboid.SetMapper(my_cuboidMapper)

# same (source, mapper, actor) for sphere
my_sphereSource = vtk.vtkSphereSource()
my_sphereSource.SetRadius(0.04)
my_sphereSource.SetThetaResolution(10)  # discretization
my_sphereSource.SetPhiResolution(10)

# mapper
my_sphereMapper = vtk.vtkPolyDataMapper()
my_sphereMapper.SetInputConnection(my_sphereSource.GetOutputPort())

# actor
my_sphere = vtk.vtkLODActor()
my_sphere.SetMapper(my_sphereMapper)


# create renderer
ren = vtk.vtkRenderer()

# add actors to renderer
ren.AddActor(my_cuboid)
ren.AddActor(my_sphere)

# create RenderWindow and RenderWindowInteractor
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(ren)
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

# background color and window size
ren.SetBackground(0.1, 0.2, 0.4)
renWin.SetSize(800, 800)

# select a more convenient interactor style and initialize
iren.SetInteractorStyle(vtk.vtkInteractorStyleTrackballCamera())
iren.Initialize()

import vtk.util.colors as colors
my_sphere.GetProperty().SetColor(colors.red)
my_cuboid.GetProperty().SetOpacity(0.5)

my_sphere.SetPosition(0.5, 0, 0)
my_cuboid.RotateWXYZ(90, 0, 1, 0)


# open window
iren.Start()

# shut down after window was closed (by mouse or by "q" key)
iren.GetRenderWindow().Finalize()
















# put this before `iren.start()`


# my_sphere.SetPosition(0.5,0,0)
# my_cuboid.RotateWXYZ(90, 0,1,0)
