import vtk

# create cuboid source
my_cuboidSource = vtk.vtkCubeSource()
my_cuboidSource.SetXLength(0.3)
my_cuboidSource.SetYLength(0.1)
my_cuboidSource.SetZLength(0.1)

# create and connect cuboid mapper
my_cuboidMapper = vtk.vtkPolyDataMapper()
my_cuboidMapper.SetInputConnection(my_cuboidSource.GetOutputPort())

# create and connect cuboid actor
my_cuboid = vtk.vtkLODActor()
my_cuboid.SetMapper(my_cuboidMapper)

# same (source, mapper, actor) for sphere
my_sphereSource = vtk.vtkSphereSource()
my_sphereSource.SetRadius(0.04)
my_sphereSource.SetThetaResolution(10) # discretisation
my_sphereSource.SetPhiResolution(10)

# mapper
my_sphereMapper = vtk.vtkPolyDataMapper()
my_sphereMapper.SetInputConnection(my_sphereSource.GetOutputPort())

# actor
my_sphere = vtk.vtkLODActor()
my_sphere.SetMapper(my_sphereMapper)


# create renderer
ren = vtk.vtkRenderer()

# add actors to renderer
ren.AddActor(my_cuboid)
ren.AddActor(my_sphere)

# create RenderWindow and RenderWindowInteractor
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(ren)
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

# background color and window size
ren.SetBackground(0.1, 0.2, 0.4)
renWin.SetSize(1000, 1000)

# select a more convenient interactor style and initialize
iren.SetInteractorStyle(vtk.vtkInteractorStyleTrackballCamera())
iren.Initialize()


# displacement via poke matrix

# create 4x4 unit matrix as numpy array
# leave first three columns unchanged: unit matrix -> no rotation

# overwrite first 3 entries of last column with displacent vector
# 1 in x direction, 2 in y direction, 2.8 in z-direction
import numpy as np
P = np.eye(4)  # 4x4 unit matrix as numpy array
P[:-1, -1] = np.array([1, 2, 2.8])

import ipydex
# ipydex.IPS()


# vtk does not understand numpy datatypes directly
# create empty vtk matrix and copy values from P
poke_matrix = vtk.vtkMatrix4x4()
vtk.vtkMatrix4x4.DeepCopy(poke_matrix, P.flatten())
my_cuboid.PokeMatrix(poke_matrix)


# open window
iren.Start()

# shut down after window was closed (by mouse or by "q" key)
iren.GetRenderWindow().Finalize()
