import vtk
from vtk.util.colors import *


from functions import calcPositions, setPokeMatrix
from model import calcStep


# -------------------------------------------------------------------------------

# cuboid for the trolley with dimension x=0.3, etc.

# trolley_source = vtk.vtkCubeSource()
# trolley_source.SetXLength(0.3)
# trolley_source.SetYLength(0.1)
# trolley_source.SetZLength(0.1)


# taks 10:
# load geometry from stl file
trolley_source = vtk.vtkSTLReader()
trolley_source.SetFileName('../data/trolley.stl')

trolley_mapper = vtk.vtkPolyDataMapper()
trolley_mapper.SetInputConnection(trolley_source.GetOutputPort())

trolley_actor = vtk.vtkLODActor()
trolley_actor.SetMapper(trolley_mapper)


# task 10:
hook_source = vtk.vtkSTLReader()
hook_source.SetFileName('../data/hook.stl')

hook_mapper = vtk.vtkPolyDataMapper()
hook_mapper.SetInputConnection(hook_source.GetOutputPort())

hook_actor = vtk.vtkLODActor()
hook_actor.SetMapper(hook_mapper)

# -------------------------------------------------------------------------------

# cuboid for the load
load_source = vtk.vtkCubeSource()
load_source.SetXLength(0.1)
load_source.SetYLength(0.1)
load_source.SetZLength(0.1)

load_mapper = vtk.vtkPolyDataMapper()
load_mapper.SetInputConnection(load_source.GetOutputPort())

load_actor = vtk.vtkLODActor()
load_actor.SetMapper(load_mapper)


# -------------------------------------------------------------------------------

# task 9:
# cylinder for the rope
rope_source = vtk.vtkCylinderSource()
rope_source.SetRadius(0.01)
rope_source.SetHeight(0.5)


rope_mapper = vtk.vtkPolyDataMapper()
rope_mapper.SetInputConnection(rope_source.GetOutputPort())

rope_actor = vtk.vtkLODActor()
rope_actor.SetMapper(rope_mapper)


# -------------------------------------------------------------------------------

# create Renderer, RenderWindow and RenderWindowInteractor
ren = vtk.vtkRenderer()
renWin = vtk.vtkRenderWindow()
renWin.AddRenderer(ren)
iren = vtk.vtkRenderWindowInteractor()
iren.SetRenderWindow(renWin)

# add actors to renderer
ren.AddActor(trolley_actor)
ren.AddActor(hook_actor)
ren.AddActor(load_actor)
ren.AddActor(rope_actor)

# background color and window size
ren.SetBackground(0.1, 0.2, 0.4)
renWin.SetSize(1000, 1000)

# select a more convenient interactor style and initialize
iren.SetInteractorStyle(vtk.vtkInteractorStyleTrackballCamera())
iren.Initialize()


# -------------------------------------------------------------------------------


def updateScene(*args):
    """
    This function recalculates the system and updates the scene.
    """

    # Calculate a simulation step
    t, y = calcStep()

    # Calculate positions and orientations of the bodies
    r_trolley, R_trolley, r_last, R_last, r_seil, R_seil = calcPositions(y)

    # update bodies
    setPokeMatrix(trolley_actor, r_trolley, R_trolley)
    setPokeMatrix(hook_actor, r_trolley, R_trolley)
    setPokeMatrix(load_actor, r_last, R_last)
    setPokeMatrix(rope_actor, r_seil, R_seil)

    # re-render the image
    renWin.Render()


# -------------------------------------------------------------------------------


# Create the timer. The updateScene function will be called every 20ms automatically
iren.AddObserver('TimerEvent', updateScene)
iren.CreateRepeatingTimer(20)

# open the window
iren.Start()

# shut down after window was closed (by mouse or by "q" key)
iren.GetRenderWindow().Finalize()
