
"""
This is the main program.
"""

# Translation will be provided soon.

import PyQt5.QtCore as QtCore
import PyQt5.QtGui as QtGui
import PyQt5.QtWidgets as QtWidgets

import sys

from model import rhs
from scipy.integrate import solve_ivp, odeint
import numpy as np

import matplotlib
matplotlib.use('Qt5Agg') # Backend explizit festlegen

import matplotlib.pyplot as plt


# QApplication Instanz wird immer benötigt (sys.argv erst mal hinnehmen)
app = QtWidgets.QApplication(sys.argv)



#-------------------------------------------------------------------------------


# Dialog erzeugen
dialog = QtWidgets.QDialog()


# task 11.1.1
# Label und Eingabefelder
length_label = QtWidgets.QLabel('pendulum length', dialog)
length_edit = QtWidgets.QLineEdit('1.2', dialog)


mass1_label = QtWidgets.QLabel('mass carriage', dialog)
mass1_edit = QtWidgets.QLineEdit('0.8', dialog)

mass2_label = QtWidgets.QLabel('mass pendulum load', dialog)
mass2_edit = QtWidgets.QLineEdit('0.5', dialog)

# task 11.1.2

step_size_label = QtWidgets.QLabel('simulation step size', dialog)
step_size_edit = QtWidgets.QLineEdit('0.01', dialog)

duration_label = QtWidgets.QLabel('simulation duration', dialog)
duration_edit = QtWidgets.QLineEdit('10.0', dialog)

# task 11.1.3
# Buttons
simulation_button = QtWidgets.QPushButton('Simulate', dialog)
exit_button = QtWidgets.QPushButton('Exit', dialog)

exit_button.clicked.connect(dialog.close)


# task 11.1.4
# Ausrichtung anpassen
layout = QtWidgets.QGridLayout()
layout.addWidget(length_label, 0, 0)  # widget, row, column
layout.addWidget(length_edit, 0, 1)
layout.addWidget(mass1_label, 1, 0)
layout.addWidget(mass1_edit, 1, 1)
layout.addWidget(mass2_label, 2, 0)
layout.addWidget(mass2_edit, 2, 1)
layout.addWidget(step_size_label, 3, 0)
layout.addWidget(step_size_edit, 3, 1)
layout.addWidget(duration_label, 4, 0)
layout.addWidget(duration_edit, 4, 1)

layout.addWidget(simulation_button, 5, 1, QtCore.Qt.AlignRight)
layout.addWidget(exit_button, 6, 1, QtCore.Qt.AlignRight)

dialog.setLayout(layout)

# task 11.1.5
# zulässige Zeichen beschränken - hier nur Zahlen eingeben
length_edit.setValidator(QtGui.QDoubleValidator(length_edit))
mass1_edit.setValidator(QtGui.QDoubleValidator(mass1_edit))
mass2_edit.setValidator(QtGui.QDoubleValidator(mass2_edit))
step_size_edit.setValidator(QtGui.QDoubleValidator(step_size_edit))
duration_edit.setValidator(QtGui.QDoubleValidator(duration_edit))


# task 11.1.6
length_edit.setAlignment(QtCore.Qt.AlignRight)
mass1_edit.setAlignment(QtCore.Qt.AlignRight)
mass2_edit.setAlignment(QtCore.Qt.AlignRight)
step_size_edit.setAlignment(QtCore.Qt.AlignRight)
duration_edit.setAlignment(QtCore.Qt.AlignRight)


# Focus auf Exit
#exit_button.setFocus()



import configparser
def saveFile():
    """
    Dialog zur Auswahl einer Datei öffnen und die aktuellen Parameter darin
    abspeichern
    """

    # Dialog für Dateiname (gibt ein 2-Tupel zurück), siehe Folie 9
    filename, type_filter = QtWidgets.QFileDialog.getSaveFileName()

    if filename == "":
        return  # wenn "Abbrechen"/"Cancel" gedrückt wurde -> nichts tun

    c = configparser.ConfigParser()
    c.add_section ("Parameter")
    c.set("Parameter", "m1", str(mass1_edit.text()))
    c.set("Parameter", "m2", str(mass2_edit.text()))
    c.set("Parameter", "l", str(length_edit.text()))

    # Configfile schreiben
    with open(filename, 'w') as fid:
        c.write(fid)


def openFile():
    """
    Öffnet eine ini-Datei und liest die gespeicherten Daten
    """

    # Dialog für Dateiname (gibt ein 2-Tupel zurück), siehe Folie 9
    filename, type_filter = QtWidgets.QFileDialog.getOpenFileName()

    if filename == "":
        return  # wenn "Abbrechen"/"Cancel" gedrückt wurde -> nichts tun

    # Configparser anlegen
    c = configparser.ConfigParser()
    c.read(str(filename))

    mass1_edit.setText(c.get("Parameter", "m1"))
    mass2_edit.setText(c.get("Parameter", "m2"))
    length_edit.setText(c.get("Parameter", "l"))


def simulate():
    """
    Diese Funktion liest die Parameter aus allen LineEdits, konvertiert sie in
    floats und führt damit die Simulation aus. Anschließend werden die
    Ergebnisse mit matplotlib dargestellt. Die Startwerte der Simulation sind
    hier noch statisch vorgegeben (Hart kodiert).
    """

    # Werte holen
    m1 = float(mass1_edit.text())
    m2 = float(mass2_edit.text())
    l = float(length_edit.text())
    step = float(step_size_edit.text())

    duration = float(duration_edit.text())


    # ...

    # Zeitachse anlegen (np.linspace)
    t = np.arange(0, duration, step)

    # Simulation ausführen, siehe Aufgabenstellung
    y0 = [0, 0.5, 0, 0]
    res = odeint( rhs, y0, t, args=(m1, m2, l) )


    # unpack the simulation result
    x, phi, xdot, phidot = res.T


    # Ergebnisse plotten
    fig = plt.figure()

    # Hier muss etwas getrickst werden: wir legen einen neuen Dialog an, auf den
    # matplotlib zeichnet. Der plot_dialog hat unseren Hauptdialog als parent und
    # ist "nicht modal". Damit können wir beliebig viele Ergebnisfenster parallel
    # darstellen und Simulationsergebnisse vergleichen.
    # Mehr Infos: https://en.wikipedia.org/wiki/Modal_window
    plot_dialog = QtWidgets.QDialog(dialog)
    fig.canvas.parent().setParent(plot_dialog)

    # Ergebnisse für Laufkatze
    ax1 = fig.add_subplot(2, 1, 1)
    ax1.plot(t, x, label="x")
    ax1.plot(t, xdot, label="xdot")
    ax1.legend()


    ax2 = fig.add_subplot(2, 1, 2)
    # Ergebnisse für Last
    ax2.plot(t, phi, label="phi")
    ax2.plot(t, phidot, label="phidot")
    ax2.legend()


    # Hier wird jetzt der Dialog angezeigt und nicht mehr die show-Funktion von
    # matplotlib aufgerufen!
    plot_dialog.show()


# task 11.1.7
simulation_button.clicked.connect(simulate)

# task 11.2.1

open_button = QtWidgets.QPushButton('Open', dialog)
save_button = QtWidgets.QPushButton('Save', dialog)

open_button.clicked.connect(openFile)
save_button.clicked.connect(saveFile)

layout.addWidget(open_button, 7, 1, QtCore.Qt.AlignRight)
layout.addWidget(save_button, 8, 1, QtCore.Qt.AlignRight)


#-------------------------------------------------------------------------------

# Dialog anzeigen
dialog.exec_()
